@@ -82,7 +82,8 @@
  */
 
 #define S_F_SINCE_BOOT		0x00000001
-#define S_F_SA_ROTAT      	0x00000002
+#define S_F_SA_ROTAT      	0x00000002	/* Only used by sadc */
+#define S_F_DEV_SID		0x00000002	/* Only used by sar/sadf */
 #define S_F_DEV_PRETTY		0x00000004
 #define S_F_FORCE_FILE		0x00000008
 #define S_F_INTERVAL_SET	0x00000010
@@ -117,6 +118,7 @@
 
 #define WANT_SINCE_BOOT(m)		(((m) & S_F_SINCE_BOOT)   == S_F_SINCE_BOOT)
 #define WANT_SA_ROTAT(m)		(((m) & S_F_SA_ROTAT)     == S_F_SA_ROTAT)
+#define USE_STABLE_ID(m)		(((m) & S_F_DEV_SID)      == S_F_DEV_SID)
 #define USE_PRETTY_OPTION(m)		(((m) & S_F_DEV_PRETTY)   == S_F_DEV_PRETTY)
 #define FORCE_FILE(m)			(((m) & S_F_FORCE_FILE)   == S_F_FORCE_FILE)
 #define INTERVAL_SET(m)			(((m) & S_F_INTERVAL_SET) == S_F_INTERVAL_SET)
@@ -186,34 +188,35 @@
 
 /* Keywords */
 #define K_A_NULL	"A_NULL"
-#define K_XALL		"XALL"
-#define K_SUM		"SUM"
+#define K_CPU		"CPU"
 #define K_DEV		"DEV"
 #define K_EDEV		"EDEV"
+#define K_EICMP		"EICMP"
+#define K_EICMP6	"EICMP6"
+#define K_EIP		"EIP"
+#define K_EIP6		"EIP6"
+#define K_ETCP		"ETCP"
+#define K_FAN		"FAN"
+#define K_FC		"FC"
+#define K_FREQ		"FREQ"
+#define K_ICMP		"ICMP"
+#define K_ICMP6		"ICMP6"
+#define K_IN		"IN"
+#define K_IP		"IP"
+#define K_IP6		"IP6"
+#define K_MOUNT		"MOUNT"
 #define K_NFS		"NFS"
 #define K_NFSD		"NFSD"
+#define K_SID		"SID"
 #define K_SOCK		"SOCK"
-#define K_IP		"IP"
-#define K_EIP		"EIP"
-#define K_ICMP		"ICMP"
-#define K_EICMP		"EICMP"
-#define K_TCP		"TCP"
-#define K_ETCP		"ETCP"
-#define K_UDP		"UDP"
 #define K_SOCK6		"SOCK6"
-#define K_IP6		"IP6"
-#define K_EIP6		"EIP6"
-#define K_ICMP6		"ICMP6"
-#define K_EICMP6	"EICMP6"
-#define K_UDP6		"UDP6"
-#define K_CPU		"CPU"
-#define K_FAN		"FAN"
-#define K_TEMP		"TEMP"
-#define K_IN		"IN"
-#define K_FREQ		"FREQ"
-#define K_MOUNT		"MOUNT"
-#define K_FC		"FC"
 #define K_SOFT		"SOFT"
+#define K_SUM		"SUM"
+#define K_TCP		"TCP"
+#define K_TEMP		"TEMP"
+#define K_UDP		"UDP"
+#define K_UDP6		"UDP6"
+#define K_XALL		"XALL"
 
 #define K_INT		"INT"
 #define K_DISK		"DISK"
@@ -362,17 +365,49 @@ struct svg_hdr_parm {
  * 	|
  * 	|--                         --|
  * 	|                             |
- * 	| file_activity structure     | * sa_act_nr
+ * 	| file_activity structure     | x file_header:sa_act_nr
  * 	|                             |
- * 	|--                         --|
+ * 	|---------                  --|
+ *		|
+ * 		| extra_desc structure (exists only if file_header:extra_next != 0)
+ * 		|
+ * 		|--                         --|
+ *		|                             |
+ * 		| unknown extra structure(s)  | x extra_desc:extra_nr
+ * 		|                             |
+ * 		|--                         --|
+ *		|
+ * 		| (extra_desc structure (exists only if previous extra_desc:extra_next != 0))
+ * 		|
+ * 		|--
+ *		|
+ * 		| (...)
+ * 		|
+ * 	|---------                  --|
  * 	|                             |
  * 	| record_header structure     |
  * 	|                             |
- * 	|--                           |
+ * 	|---------                  --|
+ *		|
+ * 		| extra_desc structure (exists only if record_header:extra_next != 0)
+ * 		|
+ * 		|--                         --|
+ *		|                             |
+ * 		| unknown extra structure(s)  | x extra_desc:extra_nr
+ * 		|                             |
+ * 		|--                         --|
+ *		|
+ * 		| (extra_desc structure (exists only if previous extra_desc:extra_next != 0))
+ * 		|
+ * 		|--
+ *		|
+ * 		| (...)
+ * 		|
+ * 	|---------                  --|
  * 	|(__nr_t)                     |
  * 	|--                           |
  * 	|                             |
- * 	| Statistics structure(s)     | * <count>
+ * 	| Statistics structure(s)     | x <count>
  * 	|                             |
  * 	|--                           |
  * 	|(__nr_t)                     |
@@ -399,6 +434,12 @@ struct svg_hdr_parm {
  * 2 for 1 CPU and SMP kernel (CPU "all" and CPU 0), etc.
  * Of course we display the real number of CPU (e.g. "1" for 1 CPU and SMP
  * kernel) with the LINUX RESTART message.
+ *
+ * If the record_header's type is R_EXTRA* then we find only a list of extra
+ * structures following the record_header structure but no statistics ones.
+ * Note that extra structures may exist for all record_header types
+ * (R_STATS, R_COMMENT, R_RESTART...). For R_COMMENT and R_RESTART records,
+ * the extra structures will be found after the comment or the number of CPU.
  ***************************************************************************
  */
 
@@ -511,6 +552,10 @@ struct file_header {
 	 */
 	unsigned int act_size;
 	unsigned int rec_size;
+	/*
+	 * TRUE if an extra_desc structure exists.
+	 */
+	unsigned int extra_next;
 	/*
 	 * Current day and month.
 	 * No need to save DST (Daylight Saving Time) flag, since it is not taken
@@ -544,7 +589,7 @@ struct file_header {
 #define FILE_HEADER_SIZE	(sizeof(struct file_header))
 #define FILE_HEADER_ULL_NR	1	/* Nr of unsigned long long in file_header structure */
 #define FILE_HEADER_UL_NR	1	/* Nr of unsigned long in file_header structure */
-#define FILE_HEADER_U_NR	11	/* Nr of [unsigned] int in file_header structure */
+#define FILE_HEADER_U_NR	12	/* Nr of [unsigned] int in file_header structure */
 /* The values below are used for sanity check */
 #define MIN_FILE_HEADER_SIZE	0
 #define MAX_FILE_HEADER_SIZE	8192
@@ -601,6 +646,37 @@ struct file_activity {
 #define FILE_ACTIVITY_UL_NR	0	/* Nr of unsigned long in file_activity structure */
 #define FILE_ACTIVITY_U_NR	9	/* Nr of [unsigned] int in file_activity structure */
 
+/*
+ * Description of an extra structure.
+ * The composition of this structure should not change in time.
+ */
+struct extra_desc {
+	/*
+	 * Number of extra structures to read.
+	 */
+	unsigned int extra_nr;
+	/*
+	 * Size of an extra structure.
+	 */
+	unsigned int extra_size;
+	/*
+	 * TRUE if another extra_desc structure exists after
+	 * all the extra structures.
+	 */
+	unsigned int extra_next;
+	/*
+	 * Description of an extra structure
+	 * (nr of "long long", nr of "long" and nr of "int").
+	 */
+	unsigned int extra_types_nr[3];
+};
+
+#define EXTRA_DESC_SIZE		(sizeof(struct extra_desc))
+#define EXTRA_DESC_ULL_NR	0	/* Nr of unsigned long long in extra_desc structure */
+#define EXTRA_DESC_UL_NR	0	/* Nr of unsigned long in extra_desc structure */
+#define EXTRA_DESC_U_NR		6	/* Nr of [unsigned] int in extra_desc structure */
+#define MAX_EXTRA_NR		8192
+#define MAX_EXTRA_SIZE		1024
 
 /* Record type */
 /*
@@ -624,6 +700,12 @@ struct file_activity {
  * a comment.
  */
 #define R_COMMENT	4
+/*
+ * R_EXTRA* records means that extra structures are following current
+ * record_header structure, but no statistics structures.
+ */
+#define R_EXTRA_MIN	5
+#define R_EXTRA_MAX	15
 
 /* Maximum length of a comment */
 #define MAX_COMMENT_LEN	64
@@ -638,6 +720,10 @@ struct record_header {
 	 * Timestamp (number of seconds since the epoch).
 	 */
 	unsigned long long ust_time;
+	/*
+	 * TRUE if an extra_desc structure exists.
+	 */
+	unsigned int extra_next;
 	/*
 	 * Record type: R_STATS, R_RESTART,...
 	 */
@@ -655,7 +741,7 @@ struct record_header {
 #define MAX_RECORD_HEADER_SIZE	512	/* Used for sanity check */
 #define RECORD_HEADER_ULL_NR	2	/* Nr of unsigned long long in record_header structure */
 #define RECORD_HEADER_UL_NR	0	/* Nr of unsigned long in record_header structure */
-#define RECORD_HEADER_U_NR	0	/* Nr of unsigned int in record_header structure */
+#define RECORD_HEADER_U_NR	1	/* Nr of unsigned int in record_header structure */
 
 
 /*
@@ -1338,6 +1424,8 @@ void print_collect_error
 	(void);
 int set_default_file
 	(char *, int, int);
+int skip_extra_struct
+	(int, int, int);
 int write_all
 	(int, const void *, int);
 
@@ -1372,7 +1460,7 @@ void free_structures
 char *get_devname
 	(unsigned int, unsigned int, int);
 char *get_sa_devname
-	(unsigned int, unsigned int, unsigned int);
+	(unsigned int, unsigned int, unsigned long long [], unsigned int, unsigned int);
 void get_file_timestamp_struct
 	(unsigned int, struct tm *, struct file_header *);
 unsigned long long get_global_cpu_statistics
