@@ -52,6 +52,7 @@ extern unsigned int dm_major;
 unsigned int hdr_types_nr[] = {FILE_HEADER_ULL_NR, FILE_HEADER_UL_NR, FILE_HEADER_U_NR};
 unsigned int act_types_nr[] = {FILE_ACTIVITY_ULL_NR, FILE_ACTIVITY_UL_NR, FILE_ACTIVITY_U_NR};
 unsigned int rec_types_nr[] = {RECORD_HEADER_ULL_NR, RECORD_HEADER_UL_NR, RECORD_HEADER_U_NR};
+unsigned int extra_desc_types_nr[] = {EXTRA_DESC_ULL_NR, EXTRA_DESC_UL_NR, EXTRA_DESC_U_NR};
 unsigned int nr_types_nr[]  = {0, 0, 1};
 
 /*
@@ -1335,7 +1336,8 @@ int remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 	/* Remap [unsigned] int fields */
 	d = gtypes_nr[1] - ftypes_nr[1];
 	if (d) {
-		if (ftypes_nr[1] * UL_ALIGNMENT_WIDTH < ftypes_nr[1])
+		if (gtypes_nr[0] * ULL_ALIGNMENT_WIDTH +
+		    ftypes_nr[1] * UL_ALIGNMENT_WIDTH < ftypes_nr[1])
 			/* Overflow */
 			return -1;
 
@@ -1364,7 +1366,9 @@ int remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 	/* Remap possible fields (like strings of chars) following int fields */
 	d = gtypes_nr[2] - ftypes_nr[2];
 	if (d) {
-		if (ftypes_nr[2] * U_ALIGNMENT_WIDTH < ftypes_nr[2])
+		if (gtypes_nr[0] * ULL_ALIGNMENT_WIDTH +
+		    gtypes_nr[1] * UL_ALIGNMENT_WIDTH +
+		    ftypes_nr[2] * U_ALIGNMENT_WIDTH < ftypes_nr[2])
 			/* Overflow */
 			return -1;
 
@@ -1445,6 +1449,66 @@ int sa_fread(int ifd, void *buffer, size_t size, int mode, int oneof)
 	return 0;
 }
 
+/*
+ ***************************************************************************
+ * Skip unknown extra structures present in file.
+ *
+ * IN:
+ * @ifd		System activity data file descriptor.
+ * @endian_mismatch
+ *		TRUE if file's data don't match current machine's endianness.
+ * @arch_64	TRUE if file's data come from a 64 bit machine.
+ *
+ * RETURNS:
+ * -1 on error, 0 otherwise.
+ ***************************************************************************
+ */
+int skip_extra_struct(int ifd, int endian_mismatch, int arch_64)
+{
+	int i;
+	struct extra_desc xtra_d;
+
+	do {
+		/* Read extra structure description */
+		sa_fread(ifd, &xtra_d, EXTRA_DESC_SIZE, HARD_SIZE, UEOF_STOP);
+
+		/*
+		 * We don't need to remap as the extra_desc structure won't change,
+		 * but we may need to normalize endianness anyway.
+		 */
+		if (endian_mismatch) {
+			swap_struct(extra_desc_types_nr, &xtra_d, arch_64);
+		}
+
+		/* Check values consistency */
+		if (MAP_SIZE(xtra_d.extra_types_nr) > xtra_d.extra_size) {
+#ifdef DEBUG
+			fprintf(stderr, "%s: extra_size=%u types=%d,%d,%d\n",
+				__FUNCTION__, xtra_d.extra_size,
+				xtra_d.extra_types_nr[0], xtra_d.extra_types_nr[1], xtra_d.extra_types_nr[2]);
+#endif
+			return -1;
+		}
+
+		if ((xtra_d.extra_nr > MAX_EXTRA_NR) || (xtra_d.extra_size > MAX_EXTRA_SIZE)) {
+#ifdef DEBUG
+			fprintf(stderr, "%s: extra_size=%u extra_nr=%u\n",
+				__FUNCTION__, xtra_d.extra_size, xtra_d.extra_size);
+#endif
+			return -1;
+		}
+
+		/* Ignore current unknown extra structures */
+		for (i = 0; i < xtra_d.extra_nr; i++) {
+			if (lseek(ifd, xtra_d.extra_size, SEEK_CUR) < xtra_d.extra_size)
+				return -1;
+		}
+	}
+	while (xtra_d.extra_next);
+
+	return 0;
+}
+
 /*
  ***************************************************************************
  * Read the record header of current sample and process it.
@@ -1478,20 +1542,32 @@ int read_record_hdr(int ifd, void *buffer, struct record_header *record_hdr,
 {
 	int rc;
 
-	if ((rc = sa_fread(ifd, buffer, (size_t) file_hdr->rec_size, SOFT_SIZE, oneof)) != 0)
-		/* End of sa data file */
-		return rc;
+	do {
+		if ((rc = sa_fread(ifd, buffer, (size_t) file_hdr->rec_size, SOFT_SIZE, oneof)) != 0)
+			/* End of sa data file */
+			return rc;
 
-	/* Remap record header structure to that expected by current version */
-	if (remap_struct(rec_types_nr, file_hdr->rec_types_nr, buffer,
-			 file_hdr->rec_size, RECORD_HEADER_SIZE, b_size) < 0)
-		return 2;
-	memcpy(record_hdr, buffer, RECORD_HEADER_SIZE);
+		/* Remap record header structure to that expected by current version */
+		if (remap_struct(rec_types_nr, file_hdr->rec_types_nr, buffer,
+				 file_hdr->rec_size, RECORD_HEADER_SIZE, b_size) < 0)
+			return 2;
+		memcpy(record_hdr, buffer, RECORD_HEADER_SIZE);
 
-	/* Normalize endianness */
-	if (endian_mismatch) {
-		swap_struct(rec_types_nr, record_hdr, arch_64);
+		/* Normalize endianness */
+		if (endian_mismatch) {
+			swap_struct(rec_types_nr, record_hdr, arch_64);
+		}
+
+		/*
+		 * Skip unknown extra structures if present.
+		 * This will be done later for R_COMMENT and R_RESTART records, as extra structures
+		 * are saved after the comment or the number of CPU.
+		 */
+		if ((record_hdr->record_type != R_COMMENT) && (record_hdr->record_type != R_RESTART) &&
+		    record_hdr->extra_next && (skip_extra_struct(ifd, endian_mismatch, arch_64) < 0))
+			return 2;
 	}
+	while ((record_hdr->record_type >= R_EXTRA_MIN) && (record_hdr->record_type <= R_EXTRA_MAX)) ;
 
 	return 0;
 }
@@ -2086,6 +2162,13 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], unsigned i
 		exit(1);
 	}
 
+	/*
+	 * Check if there are some extra structures.
+	 * We will just skip them as they are unknown for now.
+	 */
+	if (file_hdr->extra_next && (skip_extra_struct(*ifd, *endian_mismatch, *arch_64) < 0))
+		goto format_error;
+
 	return;
 
 format_error:
@@ -2192,6 +2275,11 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 				return 1;
 			}
 			(*opt)++;
+			if (!strcmp(argv[*opt], K_SID)) {
+				*flags |= S_F_DEV_SID + S_F_DEV_PRETTY;
+				return 0;
+			}
+
 			if (strnlen(argv[*opt], MAX_FILE_LEN) >= MAX_FILE_LEN - 1)
 				return 1;
 
@@ -2948,6 +3036,10 @@ int print_special_record(struct record_header *record_hdr, unsigned int l_flags,
 			}
 		}
 
+		/* Ignore unknown extra structures if present */
+		if (record_hdr->extra_next && (skip_extra_struct(ifd, endian_mismatch, arch_64) < 0))
+			return 0;
+
 		if (!dp)
 			return 0;
 
@@ -2963,6 +3055,10 @@ int print_special_record(struct record_header *record_hdr, unsigned int l_flags,
 		/* Read and replace non printable chars in comment */
 		replace_nonprintable_char(ifd, file_comment);
 
+		/* Ignore unknown extra structures if present */
+		if (record_hdr->extra_next && (skip_extra_struct(ifd, endian_mismatch, arch_64) < 0))
+			return 0;
+
 		if (!dp || !DISPLAY_COMMENT(l_flags))
 			return 0;
 
@@ -3210,15 +3306,20 @@ void get_global_soft_statistics(struct activity *a, int prev, int curr,
  * IN:
  * @major	Major number of the device.
  * @minor	Minor number of the device.
+ * @wwn		WWN identifier of the device (0 if unknown).
+ * @part_nr	Partition number (0 if unknown).
  * @flags	Flags for common options and system state.
  *
  * RETURNS:
  * The name of the device.
  ***************************************************************************
  */
-char *get_sa_devname(unsigned int major, unsigned int minor, unsigned int flags)
+char *get_sa_devname(unsigned int major, unsigned int minor, unsigned long long wwn[],
+		     unsigned int part_nr, unsigned int flags)
 {
 	char *dev_name = NULL, *persist_dev_name = NULL;
+	static char sid[64];
+	char xsid[32] = "", pn[16] = "";
 
 	if (DISPLAY_PERSIST_NAME_S(flags)) {
 		persist_dev_name = get_persistent_name_from_pretty(get_devname(major, minor, TRUE));
@@ -3228,7 +3329,17 @@ char *get_sa_devname(unsigned int major, unsigned int minor, unsigned int flags)
 		dev_name = persist_dev_name;
 	}
 	else {
-		if ((USE_PRETTY_OPTION(flags)) && (major == dm_major)) {
+		if ((USE_STABLE_ID(flags)) && (wwn[0] != 0)) {
+			if (wwn[1] != 0) {
+				sprintf(xsid, "%016llx", wwn[1]);
+			}
+			if (part_nr) {
+				sprintf(pn, "-%d", part_nr);
+			}
+			snprintf(sid, sizeof(sid), "%#016llx%s%s", wwn[0], xsid, pn);
+			dev_name = sid;
+		}
+		else if ((USE_PRETTY_OPTION(flags)) && (major == dm_major)) {
 			dev_name = transform_devmapname(major, minor);
 		}
 
