@@ -48,6 +48,10 @@
 char *sccsid(void) { return (SCCSID); }
 #endif
 
+#ifdef TEST
+void int_handler(int n) { return; }
+#endif
+
 long interval = -1, count = 0;
 
 /* TRUE if data read from file don't match current machine's endianness */
@@ -246,12 +250,16 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 		if (action & IGNORE_COMMENT) {
 			/* Ignore COMMENT record */
 			if (lseek(ifd, MAX_COMMENT_LEN, SEEK_CUR) < MAX_COMMENT_LEN) {
-				perror("lseek");
 				if (oneof == UEOF_CONT)
 					return 2;
 				close(ifd);
 				exit(2);
 			}
+
+			/* Ignore unknown extra structures if present */
+			if (record_hdr[curr].extra_next && (skip_extra_struct(ifd, endian_mismatch, arch_64) < 0))
+				return 2;
+
 			if (action & SET_TIMESTAMPS) {
 				sa_get_record_timestamp_struct(flags, &record_hdr[curr],
 							       rectime);
@@ -275,6 +283,10 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 			if (!(action & DONT_READ_CPU_NR)) {
 				file_hdr.sa_cpu_nr = read_nr_value(ifd, file, file_magic,
 								   endian_mismatch, arch_64, TRUE);
+
+				/* Ignore unknown extra structures if present */
+				if (record_hdr[curr].extra_next && (skip_extra_struct(ifd, endian_mismatch, arch_64) < 0))
+					return 2;
 			}
 			if (action & SET_TIMESTAMPS) {
 				sa_get_record_timestamp_struct(flags, &record_hdr[curr], rectime);
@@ -1509,7 +1521,7 @@ int main(int argc, char **argv)
 			if (!sar_options) {
 				usage(argv[0]);
 			}
-			if (parse_sar_I_opt(argv, &opt, act)) {
+			if (parse_sar_I_opt(argv, &opt, &flags, act)) {
 				usage(argv[0]);
 			}
 		}
@@ -1799,6 +1811,11 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (USE_OPTION_A(flags)) {
+		/* Set -P ALL -I ALL if needed */
+		set_bitmaps(act, &flags);
+	}
+
 	/* sadf reads current daily data file by default */
 	if (!dfile[0]) {
 		set_default_file(dfile, day_offset, -1);
