@@ -1230,7 +1230,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 				 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1,
 				 PLAIN_OUTPUT);
 
-		/* Fill file timestmap structure (rectime) */
+		/* Fill file timestamp structure (rectime) */
 		get_file_timestamp_struct(flags, &rectime, file_hdr);
 		strftime(cur_time, sizeof(cur_time), "%Y-%m-%d", &rectime);
 		printf(_("File date: %s\n"), cur_time);
@@ -1238,7 +1238,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		if (gmtime_r((const time_t *) &file_hdr->sa_ust_time, &loc_t) != NULL) {
 			printf(_("File time: "));
 			strftime(cur_time, sizeof(cur_time), "%T", &loc_t);
-			printf("%s UTC\n", cur_time);
+			printf("%s UTC (%lld)\n", cur_time, file_hdr->sa_ust_time);
 		}
 
 		/* File composition: file_header, file_activity, record_header */
@@ -1251,6 +1251,8 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		printf("HZ = %lu\n", file_hdr->sa_hz);
 		printf(_("Number of activities in file: %u\n"),
 		       file_hdr->sa_act_nr);
+		printf(_("Extra structures available: %c\n"),
+		       file_hdr->extra_next ? 'Y' : 'N');
 
 		printf(_("List of activities:\n"));
 		fal = file_actlst;
@@ -1594,7 +1596,8 @@ __nr_t count_new_disk(struct activity *a, int curr)
 		sdc = (struct stats_disk *) ((char *) a->buf[curr] + i * a->msize);
 
 		nr += add_list_item(&(a->item_list),
-				    get_sa_devname(sdc->major, sdc->minor, flags),
+				    get_sa_devname(sdc->major, sdc->minor, sdc->wwn,
+						   sdc->part_nr, flags),
 				    MAX_DEV_LEN);
 	}
 
